%% IOF 115 - Modelagem Numerica em Oceanografia
% 1a Lista de Exercicios
% Aluno:   Danilo Rodrigues Vieira
% No. USP: 5653262

% Modelo de Advecao-Difusao 2D de vazamento continuo de poluente
% Local: Baia da Guanabara - RJ (43 00' W - 43 20' W e 22 57' S - 22 40' S)


clear; close all; clc;


%% 1) Definicao das constantes do modelo

nmax = 300;   % numero passos de tempo
dh   = 0.25;  % coeficiente de difusao (m2/s)
dx   = 100;   % espacamento de grade (m)
dt   = 600;   % passo de tempo (s)
freqplo = 20; % frequencia de plotagem

% Pergunta ao usuario qual opcao de campo de velocidade
hperg = figure('color', 'w', 'Colormap', flipud(fliplr(hot(256))), ...
    'NumberTitle', 'off', 'Name', 'Campo de velocidade', ...
    'Units', 'Normalized', 'Position', [0.3 0.3 0.4 0.4], ...
    'MenuBar', 'none', 'Toolbar', 'none');
b1 = uicontrol(hperg, 'Style', 'pushbutton', 'String', '(c): 0.2m/s para oeste', ...
    'Units', 'Normalized', 'Position', [0.1 0.1 0.35 0.35], ...
    'Callback', 'v=0;u=-0.2;close(hperg);');
b2 = uicontrol(hperg, 'Style', 'pushbutton', 'String', '(a): 0.3m/s para sul', ...
    'Units', 'Normalized', 'Position', [0.1 0.55 0.35 0.35], ...
    'Callback', 'v=-0.3;u=0;close(hperg);');
b3 = uicontrol(hperg, 'Style', 'pushbutton', 'String', '(d): 0.1m/s para leste', ...
    'Units', 'Normalized', 'Position', [0.55 0.1 0.35 0.35], ...
    'Callback', 'v=0;u=0.1;close();');
b4 = uicontrol(hperg, 'Style', 'pushbutton', 'String', '(b): 0.6m/s para norte', ...
    'Units', 'Normalized', 'Position', [0.55 0.55 0.35 0.35], ...
    'Callback', 'v=0.6;u=0;close(hperg);');
uiwait(hperg);


%% 2) Definicao da grade computacional

% Area abrangida pelo modelo
latmin = -22.9500; % 22 57'
latmax = -22.6667; % 22 40'
lonmin = -43.3333; % 43 20'
lonmax = -43.0000; % 43 00'

% Quantidade de pontos de grade
lmax = round((abs(latmin - latmax) * 60 * 1852) / dx); % pontos de grade em Y
mmax = round((abs(lonmin - lonmax) * 60 * 1852 * ...   % pontos de grade em X
    cosd(mean([latmin latmax])))/dx);

% Definicao das matrizes
cant = zeros(lmax,mmax);
catu = zeros(lmax,mmax);
cren = zeros(lmax,mmax);

% Construcao da grade lat/lon (apenas para plotagem)
lat = linspace(latmin,latmax,lmax); 
lon = linspace(lonmin,lonmax,mmax);
[LON, LAT] = meshgrid(lon,lat);


%% 3) Calculo das constantes iniciais

dx2 = 2*dx;
qh  = dh*2*dt/dx^2;


%% 4) Imposicao de condicoes iniciais

% Posicao da descarga continua (meio da baia)
ldesc = floor(lmax/2);%:ceil(lmax/2);
mdesc = floor(mmax/2);%:ceil(mmax/2);
desc  = 10; % concentracao de poluente

% Introducao do poluente no ambientete
cren(ldesc,mdesc) = cren(ldesc,mdesc) + desc;



% Cria uma nova figura e ajusta suas propriedades
fh = figure('color', 'w', 'Colormap', flipud(fliplr(hot(256))), ...
    'NumberTitle', 'off', 'Name', 'Ex.5 - Difusao e adveccao na Baia de Guanabara', ...
    'Units', 'Normalized', 'Position', [0.1 0.1 0.8 0.8], ...
    'MenuBar', 'none', 'Toolbar', 'none');

% Inicia o contador de plotagem para plotar imediatamente
contplo = 1;

% Carrega o arquivo da linha de costa
load('costa.dat');


%% 5) Inicio do laco no tempo

for n=2:nmax
    % 7) Evolucao da solucao no tempo com a formula de recorrencia
    cren(2:lmax-1,2:mmax-1) = cant(2:lmax-1,2:mmax-1)+...
        qh*(cant(3:lmax,2:mmax-1)-2*cant(2:lmax-1,2:mmax-1)+cant(1:lmax-2,2:mmax-1))...
        -v*((catu(3:lmax,2:mmax-1)-catu(1:lmax-2,2:mmax-1))/dx2)+...
        qh*(cant(2:lmax-1,3:mmax)-2*cant(2:lmax-1,2:mmax-1)+cant(2:lmax-1,1:mmax-2))...
        -u*((catu(2:lmax-1,3:mmax)-catu(2:lmax-1,1:mmax-2))/dx2);

    cren(ldesc,mdesc) = cren(ldesc,mdesc) + desc; % Introducao do poluente no ambiente
    cren(cren < 0) = 0;
    
    contplo = contplo - 1;
    if ~contplo
        contplo = freqplo;

        % 8) Apresentacao dos resultados
        h(1) = subplot(3, 3, [1 2 4 5 7 8]);
        pcolor(LON, LAT, cren);
        hold on
        plot(costa(:,1),costa(:,2), 'k', 'LineSmoothing', 'on');
        hold off
        title(['{\bf Tempo ', num2str(n*dt/3600, '%.2f'),' horas}']);
        xlabel 'Longitude (\circ)'; ylabel 'Latitude (\circ)';
        colorbar horiz;
 
        h(2) = subplot(3, 3, 6);
        lv = floor(lmax/2)-10:ceil(lmax/2)+10;
        mv = floor(mmax/2)-10:ceil(mmax/2)+10;
        pcolor(LON(lv,mv), LAT(lv,mv), cren(lv,mv));
        hold on
        plot(costa(:,1),costa(:,2), 'k', 'LineSmoothing', 'on');
        hold off
        title 'Detalhe';
        xlabel 'Longitude (\circ)'; ylabel 'Latitude (\circ)';

        set(h, 'CLim', [0 400]);
        set(findobj('type','surface'),'FaceColor','flat','EdgeColor','none');
        pause(0.02);
    end

    % 9) Transferencia de variaveis no tempo
    cant = catu;
    catu = cren;
end % 10) Retornara ao inicio at o numero total de iteracoes


% 11) Final do processamento